import pdb
from ftplib import FTP_TLS
import ssl
import os
from tqdm import tqdm


"""Connect to FTP Server for Pensieve
Prints all files in the home directory
@return FTP object set in protected mode. defaults to penseive server
"""


def connect(user, password, ip_address='pensieve.usc.edu', port=41590):
    ftp = FTP_TLS()
    ftp.ssl_version = ssl.PROTOCOL_SSLv23
    ftp.debugging = 2
    ftp.connect(ip_address, port)
    ftp.login(user, password)
    ftp.prot_p()
    ftp.retrlines('LIST home')
    return(ftp)


"""Transfer a file to a specified filepath on Pensieve, with Progress Bar
Implementation of progress bar via tqdm
@param ftp ftplib object preconnected via connect() call
@param input_filepath str filepath on local computer
@param dest_filepath str filepath on pensieve
@param block_size_bytes int size of each chunk to transfer, in bytes. i.e. 1e5
"""


def tx_with_progress(ftp, input_filepath, dest_filepath, block_size_bytes):

    filesize = os.path.getsize(input_filepath)
    with tqdm(unit='blocks', unit_scale=True, leave=False,
              miniters=1, desc='Sending', total=filesize) as tqdm_instance:
        ftp.storbinary('STOR %s' % dest_filepath, open(input_filepath, "rb"),
                       block_size_bytes,
                       callback=lambda sent: tqdm_instance.update(len(sent)))


"""Receive a file to specified filepath from Pensieve to a local computer
Progress bar has not been implemented thus far, though the filesize has been
ascertained as filesize and is printed to console to indicate the MB of the
impending file.
@param ftp ftplib object preconnected via connect() call
@param input_filepath str filepath on the pensieve server,
    e.g. home/brian/brian_scratch/my_file.csv
@param dest_filepath str local filepath,
    e.g. ~/Downloads/my_file.csv. Destination filename can diverge from
    input filename
@param block_size_bytes int size of each chunk to transfer, in bytes. i.e. 1e5
"""


def receive(ftp, input_filepath, dest_filepath, block_size_bytes):
    ftp.sendcmd("TYPE I")
    size_set_str = "SIZE %s" % input_filepath
    filesize = int(ftp.sendcmd(size_set_str).split(" ")[1])
    print("Downloading %s MB" % str(filesize / 1e6))
    # pdb.set_trace()
    lf = open(dest_filepath, "wb")

    def callback_fn(send):
        lf.write(send)

    ftp.retrbinary('RETR ' + input_filepath, callback=callback_fn)
    lf.close()



#!/usr/bin/python

from setuptools import setup
from os import path

p = path.abspath(path.dirname(__file__))
with open(path.join(p, 'README.md')) as f:
    README = f.read()
setup(name='ftpsconnector',
      version='0.1.7',
      description='Initial connector code for pensieve',
      long_description=README,
      long_description_content_type="text/markdown",
      url='http://github.com/bc/ftpsconnector',
      author='Brian Cohn',
      author_email='brian.cohn@usc.edu',
      license='MIT',
      packages=['ftpsconnector'],
      install_requires=['tqdm'],
      zip_safe=False)



from .helper_functions import *


def use_brian_login():
    user = 'brian'
    # password = open("/Users/briancohn/Desktop/password.txt").read()
    try:  
        os.environ["MY_PASSWORD"]
    except KeyError: 
        print("Please set the environment variable MY_PASSWORD")
        sys.exit(1)
    password = os.environ["MY_PASSWORD"]
    print("MARK")
    print(password)
    ftp = connect(user, password)
    return(ftp)


class TestClass(object):
    def test_working_dir_acq(self):
        ftp = use_brian_login()
        ftp.retrlines('LIST home')
        ftp.quit()
        ftp = None

    def test_upload(self):
        ftp = use_brian_login()
        # Upload big file
        p = os.path.abspath(os.path.dirname(__file__))
        input_filepath = p + "/helper_functions.py"
        destination_filepath = "home/brian_scratch/helper_functions_remote.py"
        tx_with_progress(ftp, input_filepath, destination_filepath,
                         block_size_bytes=12500000)
        ftp.quit()
        ftp = None

    def test_download(self):
        filepath_pensieve = "home/brian_scratch/helper_functions_remote.py"
        p = os.path.abspath(os.path.dirname(__file__))
        filepath_local = p + "/helper_functions_prime.py"
        if os.path.isfile(filepath_local):
            os.remove(filepath_local)

        ftp = use_brian_login()
        # Download that file back to local
        # Make sure the local file doesn't exist before the download
        # script is run.

        receive(ftp, filepath_pensieve, filepath_local,
                block_size_bytes=12500000)
        ftp.quit()
        ftp = None
        assert os.path.isfile(filepath_local)
        os.remove(filepath_local)
        assert os.path.isfile(filepath_local) is False


